@@ -2,12 +2,13 @@
 
 import warnings
 
-from .io import *  # noqa: F401,F403
-from .utils import *  # noqa: F401,F403
+from .io import *  # noqa
+from .utils import *  # noqa
 
 try:
-    from .image import *  # noqa: F401,F403
+    from .image import *  # noqa
+    from .video import *  # noqa
 except ImportError:
     warnings.warn("Please install opencv-python to enable 'nncore.image'")
 
-__version__ = '0.2.5'
+__version__ = '0.3.1'
