@@ -11,38 +11,47 @@ import torch.multiprocessing as mp
 import nncore
 
 
+def _bind_cuda_device():
+    if not torch.cuda.is_available():
+        return
+    local_rank = int(os.environ['LOCAL_RANK'])
+    assert local_rank < torch.cuda.device_count()
+    torch.cuda.set_device(local_rank)
+
+
 def _init_dist_pytorch(backend, **kwargs):
-    if torch.cuda.is_available():
-        rank = int(os.environ['RANK'])
-        num_gpus = torch.cuda.device_count()
-        torch.cuda.set_device(rank % num_gpus)
-    dist.init_process_group(backend=backend, **kwargs)
+    _bind_cuda_device()
+    dist.init_process_group(backend, **kwargs)
 
 
-def _init_dist_slurm(backend, port=29500, **kwargs):
-    proc_id = int(os.environ['SLURM_PROCID'])
-    ntasks = os.environ['SLURM_NTASKS']
-    node_list = os.environ['SLURM_NODELIST']
-    addr = getoutput('scontrol show hostname {} | head -n1'.format(node_list))
-    os.environ['MASTER_ADDR'] = addr
-    os.environ['MASTER_PORT'] = str(port)
-    os.environ['WORLD_SIZE'] = ntasks
-    os.environ['RANK'] = str(proc_id)
-    torch.cuda.set_device(proc_id % torch.cuda.device_count())
-    dist.init_process_group(backend=backend, **kwargs)
+def _init_dist_slurm(backend, **kwargs):
+    nodes = os.environ.get('SLURM_NODELIST', os.environ['SLURM_JOB_NODELIST'])
+    master_addr = getoutput(
+        'scontrol show hostname {} | head -n1'.format(nodes))
+
+    os.environ.setdefault('MASTER_ADDR', master_addr)
+    os.environ.setdefault('MASTER_PORT', '29500')
+
+    os.environ['WORLD_SIZE'] = os.environ['SLURM_NTASKS']
+    os.environ['RANK'] = os.environ['SLURM_PROCID']
+    os.environ['LOCAL_RANK'] = os.environ['SLURM_LOCALID']
+
+    _bind_cuda_device()
+    dist.init_process_group(backend, **kwargs)
 
 
 def _get_default_device(group=None):
     backend = dist.get_backend(group)
-    device = torch.device('cuda' if backend == 'nccl' else 'cpu')
+    assert backend in ('gloo', 'nccl')
+    device = torch.device('cpu' if backend == 'gloo' else 'cuda')
     return device
 
 
 def _serialize_to_tensor(data, device):
     buffer = nncore.dumps(data)
     storage = torch.ByteStorage.from_buffer(buffer)
-    data_tensor = torch.ByteTensor(storage, device=device)
-    size_tensor = torch.LongTensor([data_tensor.numel()], device=device)
+    data_tensor = torch.ByteTensor(storage).to(device)
+    size_tensor = torch.LongTensor([data_tensor.numel()]).to(device)
     return data_tensor, size_tensor
 
 
@@ -60,17 +69,16 @@ def init_dist(launcher='pytorch', backend='gloo', **kwargs):
 
     Args:
         launcher (str, optional): Launcher for the process group. Currently
-            supported launchers include ``pytorch`` and ``slurm``. Default:
+            supported launchers include ``pytorch`` and ``slurm``, Default:
             ``'pytorch'``.
-        backend (:obj:`dist.Backend` or str, optional): The distribution
+        backend (:obj:`dist.Backend` | str, optional): The distribution
             backend to use. This field should be given as a :obj:`dist.Backend`
-            object or a str (e.g. ``'gloo'``) which can also be accessed via
-            :obj:`dist.Backend` attributes. Depending on build-time
-            configurations, valid values include ``'gloo'`` and ``'nccl'``. If
-            using multiple processes per machine with ``nccl`` backend, each
-            process must have exclusive access to every GPU it uses, as
-            sharing GPUs between processes can result in deadlocks. Default:
-            ``'gloo'``.
+            object or a str which can be accessed via :obj:`dist.Backend`
+            attributes. Depending on build-time configurations, valid values
+            are ``'gloo'`` and ``'nccl'``. If using multiple processes per
+            machine with ``nccl`` backend, each process must have exclusive
+            access to every GPU it uses, as sharing GPUs between processes can
+            result in deadlocks. Default: ``'gloo'``.
     """
     assert backend in ('gloo', 'nccl')
 
@@ -100,7 +108,7 @@ def get_rank(group=None):
     Get the rank of the current process in a process group.
 
     Args:
-        group (:obj:`dist.ProcessGroup` or None, optional): The process group
+        group (:obj:`dist.ProcessGroup` | None, optional): The process group
             to use. If not specified, the default process group will be used.
             Default: ``None``.
 
@@ -109,7 +117,7 @@ def get_rank(group=None):
     """
     if not is_distributed():
         return 0
-    return dist.get_rank(group=group or dist.group.WORLD)
+    return dist.get_rank(group=group)
 
 
 def get_world_size(group=None):
@@ -117,7 +125,7 @@ def get_world_size(group=None):
     Get the world size of a process group.
 
     Args:
-        group (:obj:`dist.ProcessGroup` or None, optional): The process group
+        group (:obj:`dist.ProcessGroup` | None, optional): The process group
             to use. If not specified, the default process group will be used.
             Default: ``None``.
 
@@ -126,7 +134,7 @@ def get_world_size(group=None):
     """
     if not is_distributed():
         return 1
-    return dist.get_world_size(group=group or dist.group.WORLD)
+    return dist.get_world_size(group=group)
 
 
 def get_dist_info(group=None):
@@ -134,17 +142,17 @@ def get_dist_info(group=None):
     Get the rank of the current process and the world size of a process group.
 
     Args:
-        group (:obj:`dist.ProcessGroup` or None, optional): The process group
+        group (:obj:`dist.ProcessGroup` | None, optional): The process group
             to use. If not specified, the default process group will be used.
             Default: ``None``.
 
     Returns:
         tuple[int]: The process rank and the world size.
     """
-    if not is_distributed():
+    if is_distributed():
+        return get_rank(group=group), get_world_size(group=group)
+    else:
         return 0, 1
-    group = group or dist.group.WORLD
-    return get_rank(group=group), get_world_size(group=group)
 
 
 def is_main_process():
@@ -161,8 +169,7 @@ def synchronize(group=None):
     """
     Synchronize all processes in a process group.
     """
-    if not dist.is_available() or not dist.is_initialized() or get_world_size(
-            group=group) == 1:
+    if not is_distributed() or get_world_size(group=group) == 1:
         return
     dist.barrier(group=group)
 
@@ -174,7 +181,7 @@ def broadcast(data=None, src=0, group=None):
     Args:
         data (any, optional): Any serializable object.
         src (int, optional): The source rank. Default: ``0``.
-        group (:obj:`dist.ProcessGroup` or None, optional): The process group
+        group (:obj:`dist.ProcessGroup` | None, optional): The process group
             to use. If not specified, the default process group will be used.
             Default: ``None``.
 
@@ -213,7 +220,7 @@ def all_gather(data, group=None):
 
     Args:
         data (any): Any serializable object.
-        group (:obj:`dist.ProcessGroup` or None, optional): The process group
+        group (:obj:`dist.ProcessGroup` | None, optional): The process group
             to use. If not specified, the default process group will be used.
             Default: ``None``.
 
@@ -250,13 +257,13 @@ def gather(data, dst=0, group=None):
     Args:
         data (any): Any serializable object.
         dst (int, optional): The destination rank. Default: ``0``.
-        group (:obj:`dist.ProcessGroup` or None, optional): The process group
+        group (:obj:`dist.ProcessGroup` | None, optional): The process group
             to use. If not specified, the default process group will be used.
             Default: ``None``.
 
     Returns:
-        list or ``None``: On ``dst``, it should be a list of data gathered \
-            from each rank. Otherwise, ``None``.
+        list | None: On ``dst``, it should be a list of data gathered from \
+            each rank. Otherwise, ``None``.
     """
     rank, world_size = get_dist_info(group=group)
     if world_size == 1:
