@@ -1,22 +1,25 @@
 # Copyright (c) Ye Liu. All rights reserved.
 
 from collections import OrderedDict
-from itertools import permutations
 
 import torch
 
 import nncore
+from nncore.nn import build_model
 from .buffer import Buffer
+from .builder import build_dataloader, build_hook
 from .comm import gather, is_main_process, synchronize
-from .hooks import Hook, build_hook
+from .hooks import Hook
 from .utils import get_checkpoint, load_checkpoint
 
-_DEFAULT_STAGES = dict(
-    epochs=5,
-    optimizer=dict(type='SGD', lr=1e-2, momentum=0.9, weight_decay=1e-4),
-    lr_schedule=dict(type='iter', policy='cosine'),
-    warmup=dict(type='iter', policy='linear', steps=500, ratio=0.001),
-    validation=dict(interval=1))
+_DEFAULT_STAGES = [
+    dict(
+        epochs=5,
+        optimizer=dict(type='SGD', lr=1e-2, momentum=0.9, weight_decay=1e-4),
+        lr_schedule=dict(type='iter', policy='cosine'),
+        warmup=dict(type='iter', policy='linear', steps=500, ratio=0.001),
+        validation=dict(interval=1))
+]
 
 _DEFAULT_HOOKS = [
     'TimerHook', 'LrUpdaterHook', 'OptimizerHook', 'CheckpointHook',
@@ -28,26 +31,26 @@ _DEFAULT_HOOKS = [
                     'start_iter', 'stage', 'epoch', 'iter', 'kwargs')
 class Engine(object):
     """
-    An engine that can take over the whole training, validation and testing
+    An engine that can take over the whole training, validation, and testing
     process, with all the baby-sitting works (stage control, optimizer
     configuration, lr scheduling, checkpoint management, metrics & tensorboard
     writing, etc.) done automatically.
 
     Args:
-        model (:obj:`nn.Module`): The model to be trained or tested. The
-            :obj:`forward` method of the model should return a dict containing
-            a ``_num_samples`` field indicating the number of samples in the
-            current batch, and optionally a ``_out`` field denoting the model
-            outputs to be collected and evaluated.
-        data_loaders (dict): The potential data loaders for training,
-            validation and testing. It should be in the format of
+        model (:obj:`nn.Module` | cfg | str): The model or config of the model.
+            The :obj:`forward` method of the model should return a dict
+            containing a ``_avg_factor`` field indicating the number of
+            samples in the current batch, and optionally a ``_out`` field
+            denoting the model outputs to be collected and evaluated.
+        data_loaders (dict | str): The configs of data loaders for training,
+            validation, and testing. The dict should be in the format of
             ``dict(train=train_loader, val=val_loader, test=test_loader)``.
-        stages (list[dict], dict, optional): The stage config or list of stage
-            configs to be scheduled. Each stage config should be a dict
-            containing the following fields:
+        stages (list[dict] | dict | None, optional): The stage config or list
+            of stage configs to be scheduled. Each stage config should be a
+            dict containing the following fields:
 
             - `epochs` (int): Number of epochs in the stage.
-            - `optimizer` (:obj:`optim.Optimizer` or dict): The optimizer or \
+            - `optimizer` (:obj:`optim.Optimizer` | dict): The optimizer or \
                 an optimizer config containing the following fields:
 
                 - `type` (str): Type of the optimizer, which can be accessed \
@@ -62,7 +65,7 @@ class Engine(object):
                     updating learning rates every epoch or iteration.
                 - `policy` (str): The learning rate policy to use. Currently \
                     supported policies include ``step``, ``cosine``, ``exp``, \
-                    ``poly`` and ``inv``.
+                    ``poly``, and ``inv``.
                 - `configs for the learning rate policy, e.g.` \
                     ``target_lr=0``. Please refer to :obj:`LrUpdaterHook` for \
                     full configs.
@@ -89,13 +92,13 @@ class Engine(object):
                 - `offset` (int, optional): The number of epochs to skip \
                     before counting the interval. Default: ``0``.
 
-            Default: ``_DEFAULT_STAGES``.
-        hooks (list[:obj:`Hook`] or list[dict] or list[str], optional): The
-            list of hooks to be registered. Each hook can be represented as a
-            :obj:`Hook`, a dict or a str. Default: ``_DEFAULT_HOOKS``.
+            Default: ``None``.
+        hooks (list[:obj:`Hook` | dict | str] | None, optional): The list of
+            extra hooks to be registered. Each hook can be represented as a
+            :obj:`Hook`, a dict or a str. Default: ``None``.
         buffer_size (int, optional): Maximum size of the buffer. Default:
             ``100000``.
-        logger (:obj:`logging.Logger` or str or None, optional): The logger or
+        logger (:obj:`logging.Logger` | str | None, optional): The logger or
             name of the logger to use. Default: ``None``.
         work_dir (str, optional): Path to the working directory. If not
             specified, the default working directory will be used. Default:
@@ -114,7 +117,7 @@ class Engine(object):
         >>> # [Stage 1] Train the model for 5 epochs using Adam optimizer with
         >>> # a fixed learning rate (1e-3) and a linear warm-up policy.
         >>> # [Stage 2] Train the model for another 3 epochs using SGD with
-        >>> # momentum optimizer and a iter-based cosine learning rate
+        >>> # momentum optimizer and an iter-based cosine learning rate
         >>> # schedule. Perform validation after every training epoch.
         >>> stages = [
         ...     dict(
@@ -136,28 +139,37 @@ class Engine(object):
     def __init__(self,
                  model,
                  data_loaders,
-                 stages=_DEFAULT_STAGES,
-                 hooks=_DEFAULT_HOOKS,
+                 stages=None,
+                 hooks=None,
                  buffer_size=100000,
                  logger=None,
                  work_dir=None,
+                 seed=None,
                  **kwargs):
-        self.model = model
+        self.model = build_model(model, **kwargs)
 
-        for a, b in permutations(('val', 'test')):
-            if a in data_loaders and b not in data_loaders:
-                data_loaders[b] = data_loaders[a]
-        self.data_loaders = data_loaders
+        if 'train' not in data_loaders:
+            data_loaders = dict(train=data_loaders)
 
-        if isinstance(stages, (list, tuple)):
-            for stage in stages:
-                stage.update(kwargs)
-            self.stages = stages
-        else:
-            stages.update(kwargs)
+        for a, b in (('val', 'test'), ('test', 'val')):
+            if a not in data_loaders:
+                loader = data_loaders[b if b in data_loaders else 'train']
+                if isinstance(loader, dict):
+                    data_loaders[a] = loader.copy()
+                else:
+                    data_loaders[a] = loader
+
+        self.data_loaders = {
+            k: build_dataloader(v, seed=seed)
+            for k, v in data_loaders.items()
+        }
+
+        if isinstance(stages, dict):
             self.stages = [stages]
+        else:
+            self.stages = stages or _DEFAULT_STAGES
 
-        self.hooks = OrderedDict()
+        self.register_hook(_DEFAULT_HOOKS)
         if hooks is not None:
             self.register_hook(hooks)
 
@@ -213,8 +225,8 @@ class Engine(object):
         Register a hook or a list of hooks into the engine.
 
         Args:
-            hook (list or :obj:`Hook` or dict or str): The hook or list of
-                hooks to be registered. Each hook should be represented as a
+            hook (list | :obj:`Hook` | dict | str): The hook or list of hooks
+                to be registered. Each hook can be represented as a
                 :obj:`Hook`, a dict or a str.
             before (str, optional): Name of the hook to be inserted before. If
                 not specified, the new hook will be added to the end of hook
@@ -234,6 +246,9 @@ class Engine(object):
                 "hook must be a Hook, a dict or a str, but got '{}'".format(
                     type(hook)))
 
+        if not hasattr(self, 'hooks'):
+            self.hooks = OrderedDict()
+
         if hook.name in self.hooks:
             if overwrite:
                 keys = list(self.hooks.keys())
@@ -258,9 +273,9 @@ class Engine(object):
         Unregister a hook or a list of hooks from the engine.
 
         Args:
-            hook (list or :obj:`Hook` or str): The hook or list of hooks to be
-                unregistered. Each hook should be represented as a :obj:`Hook`
-                or a str.
+            hook (list | :obj:`Hook` | str): The hook or list of hooks to be
+                unregistered. Each hook can be represented as a :obj:`Hook` or
+                a str.
         """
         if isinstance(hook, (list, tuple)):
             for h in hook:
@@ -276,7 +291,7 @@ class Engine(object):
         Build an optimizer for the engine.
 
         Args:
-            optimizer (:obj:`optim.Optimizer` or dict): The optimizer or an
+            optimizer (:obj:`optim.Optimizer` | dict): The optimizer or an
                 optimizer config used for constructing the optimizer.
         """
         if isinstance(optimizer, torch.optim.Optimizer):
@@ -294,7 +309,7 @@ class Engine(object):
         Load checkpoint from a file or an URL.
 
         Args:
-            checkpoint (dict or str): A dict, a filename, an URL or a
+            checkpoint (dict | str): A dict, a filename, an URL or a
                 ``torchvision://<model_name>`` str indicating the checkpoint.
             strict (bool, optional): Whether to allow different params for the
                 model and checkpoint. If ``True``, raise an error when the
@@ -317,7 +332,7 @@ class Engine(object):
         Resume from a checkpoint file.
 
         Args:
-            checkpoint (dict or str): A dict, a filename or an URL indicating
+            checkpoint (dict | str): A dict, a filename or an URL indicatin
                 the checkpoint.
             strict (bool, optional): Whether to allow different params for the
                 model and checkpoint. If ``True``, raise an error when the
@@ -329,7 +344,9 @@ class Engine(object):
 
         if self.stages != checkpoint['meta']['stages']:
             self.logger.warn(
-                'Stages in the engine and checkpoint are mismatch')
+                'Stages in the engine and checkpoint are mismatch:'
+                '\n\nCurrent stages: {}\n\nCheckpoint stages: {}'.format(
+                    self.stages, checkpoint['meta']['stages']))
 
         load_checkpoint(
             self.model, checkpoint, strict=strict, logger=self.logger)
@@ -366,8 +383,7 @@ class Engine(object):
         for key, value in output.items():
             self.buffer.update(
                 key,
-                value.detach().cpu()
-                if isinstance(value, torch.Tensor) else value)
+                value.detach().cpu() if torch.is_tensor(value) else value)
 
         self._call_hook('after_train_iter')
         self._iter += 1
@@ -384,8 +400,7 @@ class Engine(object):
         for key, value in output.items():
             self.buffer.update(
                 key,
-                value.detach().cpu()
-                if isinstance(value, torch.Tensor) else value)
+                value.detach().cpu() if torch.is_tensor(value) else value)
 
         self._call_hook('after_val_iter')
 
@@ -396,13 +411,16 @@ class Engine(object):
         for key, value in output.items():
             self.buffer.update(
                 key,
-                value.detach().cpu()
-                if isinstance(value, torch.Tensor) else value)
+                value.detach().cpu() if torch.is_tensor(value) else value)
 
     def train_epoch(self):
         self._mode = 'train'
         self.model.train()
         self.data_loader = self.data_loaders[self._mode]
+
+        if callable(getattr(self.data_loader.dataset, 'set_state', None)):
+            self.data_loader.dataset.set_state(self._mode)
+
         self._call_hook('before_train_epoch')
 
         for data in self.data_loader:
@@ -415,7 +433,12 @@ class Engine(object):
         self.logger.info('Validating...')
         self._mode = 'val'
         self.model.eval()
+        self.buffer.pop('_out', None)
         self.data_loader = self.data_loaders[self._mode]
+
+        if callable(getattr(self.data_loader.dataset, 'set_state', None)):
+            self.data_loader.dataset.set_state(self._mode)
+
         self._call_hook('before_val_epoch')
 
         prog_bar = nncore.ProgressBar(len(self.data_loader))
@@ -429,8 +452,12 @@ class Engine(object):
         self.logger.info('Evaluating...')
         self._mode = 'test'
         self.model.eval()
+        self.buffer.pop('_out', None)
         self.data_loader = self.data_loaders[self._mode]
 
+        if callable(getattr(self.data_loader.dataset, 'set_state', None)):
+            self.data_loader.dataset.set_state(self._mode)
+
         prog_bar = nncore.ProgressBar(len(self.data_loader))
         for data in self.data_loader:
             self.test_iter(data)
@@ -454,7 +481,7 @@ class Engine(object):
 
         self._call_hook('before_stage')
 
-        while self.epoch_in_stage < self.cur_stage['epochs']:
+        for _ in range(self.cur_stage['epochs'] - self.epoch_in_stage):
             self.train_epoch()
             cfg = self.cur_stage.get('validation')
             if cfg is not None and 'val' in self.data_loaders:
@@ -466,20 +493,16 @@ class Engine(object):
         self._call_hook('after_stage')
         self._stage += 1
 
-    def evaluate(self, key='_out'):
+    def evaluate(self):
         """
         Perform evaluation. This methods is expected to be called after
         validation or testing.
-
-        Args:
-            key (str, optional): The buffer key of the values to be evaluated.
-                Default: ``'_out'``.
         """
-        blob = self.buffer.pop(key)
+        blob = self.buffer.pop('_out')
         blob = gather(blob)
 
         if is_main_process():
-            blob = nncore.concat_list(blob)
+            blob = nncore.concat(blob)
             output = self.data_loader.dataset.evaluate(
                 blob, logger=self.logger)
         else:
@@ -488,17 +511,17 @@ class Engine(object):
         synchronize()
         return output
 
-    def launch(self, eval_mode=False, **kwargs):
+    def launch(self, eval=False, **kwargs):
         """
         Launch the engine.
 
         Args:
-            eval_mode (bool, optional): Whether to run evaluation only.
-                Default: ``False``.
+            eval (bool, optional): Whether to run evaluation only. Default:
+                ``False``.
         """
         self._kwargs = kwargs
 
-        if eval_mode:
+        if eval:
             self.test_epoch()
             output = self.evaluate()
             self.logger.info(
