@@ -15,19 +15,19 @@ def move_to_device(data, device):
     device.
 
     Args:
-        data (dict or list or :obj:`torch.Tensor`): The tensor or collection
-            of tensors to move.
-        device (:obj:`torch.device` or str): The destination device.
+        data (dict | list | :obj:`torch.Tensor`): The tensor or collection of
+            tensors to move.
+        device (:obj:`torch.device` | str): The destination device.
 
     Returns:
-        dict or list or :obj:`torch.Tensor`: The move tensor or \
-            collection of tensors.
+        dict | list | :obj:`torch.Tensor`: The moved tensor or collection of \
+            tensors.
     """
     if isinstance(data, dict):
         return {k: move_to_device(v, device) for k, v in data.items()}
     elif isinstance(data, (list, tuple)):
         return type(data)([move_to_device(d, device) for d in data])
-    elif isinstance(data, torch.Tensor):
+    elif torch.is_tensor(data):
         return data.to(device)
     else:
         return data
@@ -157,10 +157,11 @@ def update_bn_stats_(model, data_loader, num_iters=200, **kwargs):
 def publish_model(in_file,
                   out_file,
                   keys_to_remove=['optimizer'],
+                  device='cpu',
                   hash_type='sha256'):
     """
-    Publish a model by removing needless data in a checkpoint and hash the
-    output checkpoint file.
+    Publish a model by removing needless data in the checkpoint, moving the
+    weights to the specified device, and hashing the output checkpoint file.
 
     Args:
         in_file (str): Path to the input checkpoint file.
@@ -168,11 +169,13 @@ def publish_model(in_file,
             end with ``'.pth'``.
         keys_to_remove (list[str], optional): The list of keys to be removed
             from the checkpoint. Default: ``['optimizer']``.
+        device (:obj:`torch.device` | str): The destination device. Default:
+            ``'cpu'``.
         hash_type (str, optional): Type of the hash algorithm. Currently
             supported algorithms include ``'md5'``, ``'sha1'``, ``'sha224'``,
             ``'sha256'``, ``'sha384'``, ``'sha512'``, ``'blake2b'``,
             ``'blake2s'``, ``'sha3_224'``, ``'sha3_256'``, ``'sha3_384'``,
-            ``'sha3_512'``, ``'shake_128'`` and ``'shake_256'``. Default:
+            ``'sha3_512'``, ``'shake_128'``, and ``'shake_256'``. Default:
             ``'sha256'``.
     """
     assert out_file.endswith('.pth')
@@ -182,6 +185,8 @@ def publish_model(in_file,
     for key in keys_to_remove:
         if key in checkpoint:
             del checkpoint[key]
+
+    checkpoint = move_to_device(checkpoint, device)
     torch.save(checkpoint, out_file)
 
     with open(out_file, 'rb') as f:
