@@ -2,25 +2,25 @@
 
 from .binder import bind_getter, bind_method
 from .config import CfgNode, Config
-from .data import (concat_list, is_list_of, is_seq_of, is_tuple_of, slice_list,
+from .data import (concat, flatten, is_list_of, is_seq_of, is_tuple_of, slice,
                    swap_element, to_dict_of_list, to_list_of_dict)
 from .env import collect_env_info, get_host_info, get_time_str, get_timestamp
 from .logger import get_logger, log_or_print
 from .misc import recursive
-from .path import (abs_path, base_name, cp, dir_name, expand_user, is_dir,
-                   is_file, join, ls, mkdir, mv, pure_ext, pure_name, remove,
-                   rename, split_ext, symlink)
+from .path import (abs_path, base_name, cp, dir_name, expand_user, find,
+                   is_dir, is_file, join, ls, mkdir, mv, pure_ext, pure_name,
+                   remove, rename, same_dir, split_ext, symlink)
 from .progress import ProgressBar
 from .registry import Registry, build_object
 from .timer import Timer
 
 __all__ = [
-    'bind_getter', 'bind_method', 'CfgNode', 'Config', 'concat_list',
-    'is_list_of', 'is_seq_of', 'is_tuple_of', 'slice_list', 'swap_element',
+    'bind_getter', 'bind_method', 'CfgNode', 'Config', 'concat', 'flatten',
+    'is_list_of', 'is_seq_of', 'is_tuple_of', 'slice', 'swap_element',
     'to_dict_of_list', 'to_list_of_dict', 'collect_env_info', 'get_host_info',
     'get_time_str', 'get_timestamp', 'get_logger', 'log_or_print', 'recursive',
-    'abs_path', 'base_name', 'cp', 'dir_name', 'expand_user', 'is_dir',
+    'abs_path', 'base_name', 'cp', 'dir_name', 'expand_user', 'find', 'is_dir',
     'is_file', 'join', 'ls', 'mkdir', 'mv', 'pure_ext', 'pure_name', 'remove',
-    'rename', 'split_ext', 'symlink', 'ProgressBar', 'Registry',
+    'rename', 'same_dir', 'split_ext', 'symlink', 'ProgressBar', 'Registry',
     'build_object', 'Timer'
 ]
